==> gatsby-source-drupal-cross-bundle-0.0.4.tgz: package/utils.js <==
"use strict";

const _ = require(`lodash`);

const {
  nodeFromData,
  downloadFile,
  isFileNode
} = require(`./normalize`);

const backRefsNamesLookup = new WeakMap();
const referencedNodesLookup = new WeakMap();

const handleReferences = (node, {
  getNode,
  createNodeId
}) => {
  const relationships = node.relationships;

  if (node.drupal_relationships) {
    const referencedNodes = [];

    _.each(node.drupal_relationships, (v, k) => {
      if (!v.data) return;
      const nodeFieldName = `${k}___NODE`;

      if (_.isArray(v.data)) {
        relationships[nodeFieldName] = _.compact(v.data.map(data => {
          const referencedNodeId = createNodeId(data.id);

          if (!getNode(referencedNodeId)) {
            return null;
          }

          referencedNodes.push(referencedNodeId);
          return referencedNodeId;
        }));

        const meta = _.compact(v.data.map(data => !_.isEmpty(data.meta) ? data.meta : null)); // If there's meta on the field and it's not an existing/internal one
        // create a new node's field with that meta. It can't exist on both
        // @see https://jsonapi.org/format/#document-resource-object-fields


        if (!_.isEmpty(meta) && !(k in node)) {
          node[k] = meta;
        }
      } else {
        const referencedNodeId = createNodeId(v.data.id);

        if (getNode(referencedNodeId)) {
          relationships[nodeFieldName] = referencedNodeId;
          referencedNodes.push(referencedNodeId);
        } // If there's meta on the field and it's not an existing/internal one
        // create a new node's field with that meta. It can't exist on both
        // @see https://jsonapi.org/format/#document-resource-object-fields


        if (!_.isEmpty(v.data.meta) && !(k in node)) {
          node[k] = v.data.meta;
        }
      }
    });

    delete node.drupal_relationships;
    referencedNodesLookup.set(node, referencedNodes);

    if (referencedNodes.length) {
      const nodeFieldName = `${node.internal.type}___NODE`;
      referencedNodes.forEach(nodeID => {
        const referencedNode = getNode(nodeID);

        if (!referencedNode.relationships[nodeFieldName]) {
          referencedNode.relationships[nodeFieldName] = [];
        }

        if (!referencedNode.relationships[nodeFieldName].includes(node.id)) {
          referencedNode.relationships[nodeFieldName].push(node.id);
        }

        let backRefsNames = backRefsNamesLookup.get(referencedNode);

        if (!backRefsNames) {
          backRefsNames = [];
          backRefsNamesLookup.set(referencedNode, backRefsNames);
        }

        if (!backRefsNames.includes(nodeFieldName)) {
          backRefsNames.push(nodeFieldName);
        }
      });
    }
  }

  node.relationships = relationships;
};

exports.handleReferences = handleReferences;

const handleWebhookUpdate = async ({
  nodeToUpdate,
  actions,
  cache,
  createNodeId,
  createContentDigest,
  getNode,
  reporter,
  store
}, pluginOptions) => {
  const {
    createNode
  } = actions;
  const newNode = nodeFromData(nodeToUpdate, createNodeId);
  const nodesToUpdate = [newNode];
  handleReferences(newNode, {
    getNode,
    createNodeId
  });
  const oldNode = getNode(newNode.id);

  if (oldNode) {
    // copy over back references from old node
    const backRefsNames = backRefsNamesLookup.get(oldNode);

    if (backRefsNames) {
      backRefsNamesLookup.set(newNode, backRefsNames);
      backRefsNames.forEach(backRefFieldName => {
        newNode.relationships[backRefFieldName] = oldNode.relationships[backRefFieldName];
      });
    }

    const oldNodeReferencedNodes = referencedNodesLookup.get(oldNode);
    const newNodeReferencedNodes = referencedNodesLookup.get(newNode); // see what nodes are no longer referenced and remove backRefs from them

    const removedReferencedNodes = _.difference(oldNodeReferencedNodes, newNodeReferencedNodes).map(id => getNode(id));

    nodesToUpdate.push(...removedReferencedNodes);
    const nodeFieldName = `${newNode.internal.type}___NODE`;
    removedReferencedNodes.forEach(referencedNode => {
      referencedNode.relationships[nodeFieldName] = referencedNode.relationships[nodeFieldName].filter(id => id !== newNode.id);
    }); // see what nodes are newly referenced, and make sure to call `createNode` on them

    const addedReferencedNodes = _.difference(newNodeReferencedNodes, oldNodeReferencedNodes).map(id => getNode(id));

    nodesToUpdate.push(...addedReferencedNodes);
  } else {
    // if we are inserting new node, we need to update all referenced nodes
    const newNodeReferencedNodes = referencedNodesLookup.get(newNode).map(id => getNode(id));
    nodesToUpdate.push(...newNodeReferencedNodes);
  } // download file


  if (isFileNode(newNode)) {
    await downloadFile({
      node: newNode,
      store,
      cache,
      createNode,
      createNodeId
    }, pluginOptions);
  }

  for (const node of nodesToUpdate) {
    if (node.internal.owner) {
      delete node.internal.owner;
    }

    node.internal.contentDigest = createContentDigest(node);
    createNode(node);
    reporter.log(`Updated node: ${node.id}`);
  }
};

exports.handleWebhookUpdate = handleWebhookUpdate;

==> gatsby-source-drupal-cross-bundle-1.0.1.tgz: package/utils.js <==
"use strict";

const _ = require(`lodash`);

const {
  nodeFromData,
  downloadFile,
  isFileNode
} = require(`./normalize`);

const backRefsNamesLookup = new WeakMap();
const referencedNodesLookup = new WeakMap();

const handleReferences = (node, {
  getNode,
  createNodeId
}) => {
  const relationships = node.relationships;

  if (node.drupal_relationships) {
    const referencedNodes = [];

    _.each(node.drupal_relationships, (v, k) => {
      if (!v.data) return;
      const nodeFieldName = `${k}___NODE`;

      if (_.isArray(v.data)) {
        relationships[nodeFieldName] = _.compact(v.data.map(data => {
          const referencedNodeId = createNodeId(`cross-bundle-${data.id}`);

          if (!getNode(referencedNodeId)) {
            return null;
          }

          referencedNodes.push(referencedNodeId);
          return referencedNodeId;
        }));

        const meta = _.compact(v.data.map(data => !_.isEmpty(data.meta) ? data.meta : null)); // If there's meta on the field and it's not an existing/internal one
        // create a new node's field with that meta. It can't exist on both
        // @see https://jsonapi.org/format/#document-resource-object-fields


        if (!_.isEmpty(meta) && !(k in node)) {
          node[k] = meta;
        }
      } else {
        const referencedNodeId = createNodeId(`cross-bundle-${v.data.id}`);

        if (getNode(referencedNodeId)) {
          relationships[nodeFieldName] = referencedNodeId;
          referencedNodes.push(referencedNodeId);
        } // If there's meta on the field and it's not an existing/internal one
        // create a new node's field with that meta. It can't exist on both
        // @see https://jsonapi.org/format/#document-resource-object-fields


        if (!_.isEmpty(v.data.meta) && !(k in node)) {
          node[k] = v.data.meta;
        }
      }
    });

    delete node.drupal_relationships;
    referencedNodesLookup.set(node, referencedNodes);

    if (referencedNodes.length) {
      const nodeFieldName = `${node.internal.type}___NODE`;
      referencedNodes.forEach(nodeID => {
        const referencedNode = getNode(nodeID);

        if (!referencedNode.relationships[nodeFieldName]) {
          referencedNode.relationships[nodeFieldName] = [];
        }

        if (!referencedNode.relationships[nodeFieldName].includes(node.id)) {
          referencedNode.relationships[nodeFieldName].push(node.id);
        }

        let backRefsNames = backRefsNamesLookup.get(referencedNode);

        if (!backRefsNames) {
          backRefsNames = [];
          backRefsNamesLookup.set(referencedNode, backRefsNames);
        }

        if (!backRefsNames.includes(nodeFieldName)) {
          backRefsNames.push(nodeFieldName);
        }
      });
    }
  }

  node.relationships = relationships;
};

exports.handleReferences = handleReferences;

const handleWebhookUpdate = async ({
  nodeToUpdate,
  actions,
  cache,
  createNodeId,
  createContentDigest,
  getNode,
  reporter,
  store
}, pluginOptions) => {
  const {
    createNode
  } = actions;
  const newNode = nodeFromData(nodeToUpdate, createNodeId);
  const nodesToUpdate = [newNode];
  handleReferences(newNode, {
    getNode,
    createNodeId
  });
  const oldNode = getNode(newNode.id);

  if (oldNode) {
    // copy over back references from old node
    const backRefsNames = backRefsNamesLookup.get(oldNode);

    if (backRefsNames) {
      backRefsNamesLookup.set(newNode, backRefsNames);
      backRefsNames.forEach(backRefFieldName => {
        newNode.relationships[backRefFieldName] = oldNode.relationships[backRefFieldName];
      });
    }

    const oldNodeReferencedNodes = referencedNodesLookup.get(oldNode);
    const newNodeReferencedNodes = referencedNodesLookup.get(newNode); // see what nodes are no longer referenced and remove backRefs from them

    const removedReferencedNodes = _.difference(oldNodeReferencedNodes, newNodeReferencedNodes).map(id => getNode(id));

    nodesToUpdate.push(...removedReferencedNodes);
    const nodeFieldName = `${newNode.internal.type}___NODE`;
    removedReferencedNodes.forEach(referencedNode => {
      referencedNode.relationships[nodeFieldName] = referencedNode.relationships[nodeFieldName].filter(id => id !== newNode.id);
    }); // see what nodes are newly referenced, and make sure to call `createNode` on them

    const addedReferencedNodes = _.difference(newNodeReferencedNodes, oldNodeReferencedNodes).map(id => getNode(id));

    nodesToUpdate.push(...addedReferencedNodes);
  } else {
    // if we are inserting new node, we need to update all referenced nodes
    const newNodeReferencedNodes = referencedNodesLookup.get(newNode).map(id => getNode(id));
    nodesToUpdate.push(...newNodeReferencedNodes);
  } // download file


  if (isFileNode(newNode)) {
    await downloadFile({
      node: newNode,
      store,
      cache,
      createNode,
      createNodeId
    }, pluginOptions);
  }

  for (const node of nodesToUpdate) {
    if (node.internal.owner) {
      delete node.internal.owner;
    }

    node.internal.contentDigest = createContentDigest(node);
    createNode(node);
    reporter.log(`Updated node: ${node.id}`);
  }
};

exports.handleWebhookUpdate = handleWebhookUpdate;
